#ifndef IOCTLMACRO_H
#define IOCTLMACRO_H

#include "IoControlIds.h"
#include <assert.h>
#include <string>
#include "medbglog.h"



namespace mebius_daq{
class IDeviceInfo;
}

using namespace mebius::ioctl;

using mebase::S_MEB_OK;
using mebase::E_MEB_INVALIDARG;
using mebase::MEBRESULT;



template <class BUS>
inline MEBRESULT IoCmd_SetNetworkSettings(BUS a_bus, MEB_IOCTL_NET_CONFIG* a_pSettings)
{
	if(!a_bus || !a_pSettings) {
		return E_MEB_INVALIDARG;
	}
	
	MEBRESULT mres = mebase::E_MEB_FAIL;
	mebase::ULONG nReaded = 0;
	MEBRESULT mr = a_bus->IoControl(
			IOCTL_MEASTASK_SET_NET_CONFIG, 
			a_pSettings, 
			sizeof (MEB_IOCTL_NET_CONFIG), 
			&mres, sizeof mres, 
			&nReaded, 0
		);
	return mres;
}


template <class BUS>
inline MEBRESULT IoCmd_GetNetworkSettings(BUS a_bus, MEB_IOCTL_NET_CONFIG* a_pSettings)
{
	if(!a_bus || !a_pSettings) {
		return E_MEB_INVALIDARG;
	}

	mebase::ULONG nReaded = 0;
	MEBRESULT mr = a_bus->IoControl(
			IOCTL_MEASTASK_GET_NET_CONFIG, 
			0, 0, 
			a_pSettings, sizeof (MEB_IOCTL_NET_CONFIG), 
			&nReaded, 0);
	return mr;
}

template <class BUS>
inline MEBRESULT IoCmd_CleanupDevice(BUS bus)
{
	if(!bus) {
		return E_MEB_INVALIDARG;
	}

	MEBRESULT mresCleanup;
	mebase::ULONG nReaded=0;
	MEBRESULT mr=bus->IoControl(IOCTL_MEASTASK_DO_CLEANUP, 0, 0, &mresCleanup, sizeof mresCleanup, &nReaded);
	if(MEB_FAILED(mr))
		return mr;
	return mresCleanup;
}

template <class BUS>
inline MEBRESULT IoCmd_StartDevice(BUS bus)
{
	if(!bus) {
		return E_MEB_INVALIDARG;
	}

	MEBRESULT mres;
	mebase::ULONG nReaded=0;
	MEBRESULT mr = bus->IoControl(IOCTL_MEASTASK_START, 0, 0, &mres, sizeof mres, &nReaded);
	if(MEB_FAILED(mr))
		return mr;
	return mres;
}

template <class BUS>
inline MEBRESULT IoCmd_StopDevice(BUS bus)
{
	MEBRESULT mres;
	mebase::ULONG nReaded=0;
	MEBRESULT mr=bus->IoControl(IOCTL_MEASTASK_STOP, 0, 0, &mres, sizeof mres, &nReaded);
	if(MEB_FAILED(mr))
		return mr;
	return mres;
}





template <class BUS>
inline MEBRESULT IoCmd_CreateDevice(BUS bus, const mebius_daq::IDeviceInfo* pDevInfo, mebase::ULONG* pInternalHandle)
{

	if (!bus) {
		return E_MEB_INVALIDARG;
	}

	*pInternalHandle = 0;

	MEB_IOCTL_CREATE_OBJ_RESULT objInternalHandle;
	MEB_IOCTL_DEVICE_INFO di;
	
	mestring msAddress;
	pDevInfo->GetAddress(msAddress);
	assert(std::string(msAddress).length() < MAX_ADDRESS_SIZE);
	std::string strAddress = msAddress;
	
	///      .
	/// ..   . -  
	if (strAddress[0] == '\\') {
		int pos = strAddress.find('\\', 1);
		if (pos > 0) {
			strAddress = strAddress.substr(pos);
		}
	}
	strncpy(di.address, strAddress.c_str(), MAX_ADDRESS_SIZE);
	pDevInfo->GetDeviceClassId(&di.devid);

	mebase::ULONG nReaded=0;
	MEBRESULT mr=bus->IoControl(IOCTL_MEASTASK_CREATE_DEVICE, &di, sizeof(MEB_IOCTL_DEVICE_INFO), &objInternalHandle, sizeof objInternalHandle, &nReaded);
	if(MEB_FAILED(mr))
		return mr;
	*pInternalHandle=objInternalHandle.handle;
	return objInternalHandle.mres;
}

template <class BUS>
inline MEBRESULT IoCmd_CreateChannel(BUS bus, const std::string& strAddress, mebase::ULONG* pInternalHandle)
{

	if(!bus) {
		return E_MEB_INVALIDARG;
	}

	*pInternalHandle=0;

	MEB_IOCTL_CREATE_OBJ_RESULT objInternalHandle;
	MEB_IOCTL_ADDRESS objAddress;
	
	assert(strAddress.length()<MAX_ADDRESS_SIZE);
	strncpy(objAddress.address,strAddress.c_str(),MAX_ADDRESS_SIZE);

	mebase::ULONG nReaded=0;
	MEBRESULT mr=bus->IoControl(IOCTL_MEASTASK_CREATE_CHAN, &objAddress, sizeof(objAddress), &objInternalHandle, sizeof objInternalHandle, &nReaded);
	if(MEB_FAILED(mr)){
		return mr;
	}
	*pInternalHandle=objInternalHandle.handle;
	return objInternalHandle.mres;

}

template <class BUS>
inline MEBRESULT IoCmd_CreateChannels(BUS bus, const mebase::BYTE*pBits, mebase::ULONG nChansCount)
{
	if(!bus) {
		return E_MEB_INVALIDARG;
	}


	MEBRESULT mresCmd;
	MEB_IOCTL_BITS_SET objBits;
	
	assert(nChansCount/8<MAX_BITS_FIELD_SIZE);
	memcpy(objBits.bits,pBits,nChansCount/8);

	mebase::ULONG nReaded=0;
	MEBRESULT mr=bus->IoControl(
		IOCTL_MEASTASK_CREATE_CHANNELS, 
		&objBits, sizeof(objBits), 
		&mresCmd, sizeof mresCmd, 
		&nReaded);
	if(MEB_FAILED(mr)){
		return mr;
	}
	return mresCmd;

}

template <class BUS>
inline MEBRESULT IoCmd_FinishProgramming(BUS bus,mebase::ULONG m_nSessionId)
{
	if (!bus) {
		return E_MEB_INVALIDARG;
	}
	
	mebase::ULONG nReaded = 0;
	MEBRESULT mresCmd = S_MEB_OK;
	MEBRESULT mr = bus->IoControl(
			IOCTL_MEASTASK_SET_SESSION_ID, 
			&m_nSessionId, sizeof(m_nSessionId), 
			&mresCmd, sizeof mresCmd, 
			&nReaded
		);
	if (MEB_SUCCEEDED(mr)) {
		mr = bus->IoControl(IOCTL_MEASTASK_PROGRAM, 0, 0, &mresCmd, sizeof mresCmd, &nReaded);
	}
	else {
		return mr;
	}
	if (MEB_FAILED(mr)) {
		return mr;
	}

	if (MEB_FAILED(mresCmd)) {
		return mresCmd;
	}
	else {
		return mresCmd;
	}
}


template <class BUS>
inline MEBRESULT IoCmd_ProgrammDeviceBin(BUS bus, mebase::ULONG nIH, const void* pBuffer, mebase::ULONG nBuffSize)
{
	MEBRESULT mresCmd = S_MEB_OK;
	if (!bus) {
		return E_MEB_INVALIDARG;
	}
	int sz = sizeof(mebase::ULONG)*2+nBuffSize;
	MEB_IOCTL_PROGRAMMING_BLOCK* pObjBlock = (MEB_IOCTL_PROGRAMMING_BLOCK*)new mebase::BYTE[sz];
	
	pObjBlock->handle = nIH;
	pObjBlock->size = nBuffSize;
	memcpy((mebase::BYTE*)pObjBlock + sizeof(mebase::ULONG) * 2, pBuffer, nBuffSize);

	mebase::ULONG nReaded = 0;
	MEBRESULT mr = bus->IoControl(
			IOCTL_MEASTASK_PROGRAMM_DEVICE_BIN,
			pObjBlock, sz, 
			&mresCmd, sizeof mresCmd, 
			&nReaded);
	if (MEB_FAILED(mr)) {
		return mr;
	}
	delete[] pObjBlock;
	return mresCmd;

}

template <class BUS>
inline MEBRESULT IoCmd_ZBalanceChannel(BUS bus, mebase::ULONG nIH, mebase::ULONG nChanNum, mebase::ULONG* a_pVal)
{
	if(!bus) {
		return E_MEB_INVALIDARG;
	}

	MEB_IOCTL_CREATE_OBJ_RESULT objInternalHandle;
	int sz=sizeof(mebase::ULONG)*3;
	MEB_IOCTL_PROGRAMMING_BLOCK* pObjBlock=(MEB_IOCTL_PROGRAMMING_BLOCK*)new mebase::BYTE[sz];
	
	pObjBlock->handle=nIH;
	pObjBlock->size=sizeof(mebase::ULONG);
	memcpy((mebase::BYTE*)pObjBlock+sizeof(mebase::ULONG)*2, &nChanNum, pObjBlock->size);

	mebase::ULONG nReaded=0;
	MEBRESULT mr=bus->IoControl(
		IOCTL_MEASTASK_ZBALANCE,
		pObjBlock, sz,
		&objInternalHandle, sizeof objInternalHandle,
		&nReaded);
	if(MEB_FAILED(mr)){
		return mr;
	}
	*a_pVal=objInternalHandle.handle;
	delete pObjBlock;
	return objInternalHandle.mres;
}

template <class BUS>
inline MEBRESULT IoCmd_Connect(BUS bus, mebase::ULONG nIH, const void* pBuffer, mebase::ULONG nBuffSize)
{
	if(!bus) {
		return E_MEB_INVALIDARG;
	}
	MEBRESULT mresCmd;
	int sz = sizeof(mebase::ULONG)*2 + nBuffSize;
	MEB_IOCTL_PROGRAMMING_BLOCK* pObjBlock = (MEB_IOCTL_PROGRAMMING_BLOCK*)new mebase::BYTE[sz];

	pObjBlock->handle = nIH;
	pObjBlock->size = nBuffSize;
	memcpy((mebase::BYTE*)pObjBlock+sizeof(mebase::ULONG)*2, pBuffer, nBuffSize);
	mebase::ULONG nReaded = 0;
	MEBRESULT mr=bus->IoControl(IOCTL_MEASTASK_CONNECT_DEVICE, pObjBlock, sz, &mresCmd, sizeof mresCmd, &nReaded);
	delete pObjBlock;
	if(MEB_FAILED(mr)){
		return mr;
	}
	return mresCmd;
}


template <class BUS>
inline MEBRESULT IoCmd_LoadBios(BUS bus, mebase::ULONG nIH, const void* pBuffer, mebase::ULONG nBuffSize)
{
	if(!bus) {
		return E_MEB_INVALIDARG;
	}

	MEBRESULT mresCmd;
	int sz=sizeof(mebase::ULONG)*2+nBuffSize;
	MEB_IOCTL_PROGRAMMING_BLOCK* pObjBlock=(MEB_IOCTL_PROGRAMMING_BLOCK*)new mebase::BYTE[sz];
	
	pObjBlock->handle=nIH;
	pObjBlock->size=nBuffSize;
	memcpy((mebase::BYTE*)pObjBlock+sizeof(mebase::ULONG)*2,pBuffer,nBuffSize);
	
	mebase::ULONG nReaded=0;
	MEBRESULT mr=bus->IoControl(
		IOCTL_MEASTASK_LOAD_BIOS,
		pObjBlock, sz, 
		&mresCmd, sizeof mresCmd, 
		&nReaded);
	if(MEB_FAILED(mr)){
		return mr;
	}
	delete pObjBlock;
	return mresCmd;
}



template <class BUS>
inline MEBRESULT IoCmd_QueryRamResourceStatus(BUS a_bus, mebase::ULONG a_idResource, MEB_IOCTL_RND_IO_INFO* a_pInfo)
{
	if(!a_bus || !a_pInfo) {
		return E_MEB_INVALIDARG;
	}

	MEB_IOCTL_RND_IO_INFO params;
	params.id = a_idResource;

	mebase::ULONG nReaded = 0;
	MEBRESULT mr = a_bus->IoControl(
			IOCTL_MEASTASK_QUERY_RAM_RES_STATUS, 
			&params, sizeof(MEB_IOCTL_RND_IO_INFO), 
			a_pInfo, sizeof(MEB_IOCTL_RND_IO_INFO), 
			&nReaded, 
			0
		);
	return mr;
}


template <class BUS>
inline MEBRESULT IoCmd_ReadRamResource(BUS bus, mebase::ULONG idResource, mebase::ULONGLONG nPos, void* pBuffer, mebase::ULONG nBuffSize, mebase::ULONG& nReaded)
{
	if(!bus) {
		return E_MEB_INVALIDARG;
	}

	MEBRESULT mresCmd = S_MEB_OK;
	int sz = nBuffSize + RND_IO_BLOCK_HEADER_SIZE + 8;
	
	MEB_IOCTL_RND_IO_BLOCK params;

	MEB_IOCTL_RND_IO_BLOCK* pBlock = (MEB_IOCTL_RND_IO_BLOCK*)new mebase::BYTE[sz*2];

	memset(&params, 0 , sizeof params);
	memset(pBlock, 0 , sz * 2);
	
	params.id = idResource;
	params.size = nBuffSize;
	params.pos = nPos;
	params.mode = 0;

	nReaded = 0;
	MEBRESULT mr = bus->IoControl(
		IOCTL_MEASTASK_READ_RAM_RESOURCE,
		&params, sizeof params, 
		pBlock, sz, 
		&nReaded,
		0);

	if(MEB_FAILED(mr)){
		return mr;
	}
	memcpy(pBuffer, pBlock->data, pBlock->size);
	nReaded = pBlock->size;
	delete pBlock;
	return mresCmd;
}

template <class BUS>
inline MEBRESULT IoCmd_WriteRamResource(BUS a_bus, mebase::ULONG a_idResource, mebase::ULONGLONG a_nPos, const void* a_pBuffer, mebase::ULONG a_nBuffSize, mebase::ULONG& a_nWritten)
{
	if(!a_bus) {
		return E_MEB_INVALIDARG;
	}

	MEBRESULT mresCmd = S_MEB_OK;
	int sz = a_nBuffSize + RND_IO_BLOCK_HEADER_SIZE;
	
	MEB_IOCTL_RND_IO_BLOCK result;

	MEB_IOCTL_RND_IO_BLOCK* pBlock = (MEB_IOCTL_RND_IO_BLOCK*)new mebase::BYTE[sz*2];

	memset(&result, 0 , sizeof result);
	memset(pBlock, 0 , sz * 2);
	
	pBlock->id = a_idResource;
	pBlock->size = a_nBuffSize;
	pBlock->pos = a_nPos;
	pBlock->mode = 1;
	memcpy(pBlock->data, a_pBuffer, a_nBuffSize);
	mebase::ULONG nWritten = 0;
	MEBRESULT mr = a_bus->IoControl(
		IOCTL_MEASTASK_WRITE_RAM_RESOURCE,
		pBlock, sz, 
		&result, sizeof result, 
		&nWritten,
		0);

	if(MEB_FAILED(mr)){
		return mr;
	}

	a_nWritten = result.size;
	delete pBlock;
	return mresCmd;
}


#endif
